@@ -224,7 +224,7 @@ void HttpListenerList::passListeners()
         if ((*iter)->getfd() != -1)
         {
             --count;
-            LS_INFO("Pass listener %s, copy fd %d to %d.", (*iter)->getAddrStr(),
+            LS_NOTICE("Pass listener %s, copy fd %d to %d.", (*iter)->getAddrStr(),
                     (*iter)->getfd(), startfd + count);
             dup2((*iter)->getfd(), startfd + count);
         }
@@ -401,8 +401,8 @@ const ServerAddrInfo *ServerAddrRegistry::get(const struct sockaddr *pAddr,
         if (pListener == NULL)
         {
             snprintf(sAddr, sizeof(sAddr), 
-                        (pAddr->sa_family == AF_INET6) ? "[%s]:%d" : "%s:%d", 
-                        pInfo->getAddrStr()->c_str(), GSockAddr::getPort(pAddr));
+                        (pAddr->sa_family == AF_INET6) ? "[%s]:%u" : "%s:%u",
+                        pInfo->getAddrStr()->c_str(), (unsigned)GSockAddr::getPort(pAddr));
 
             pListener = m_pListenerList->get(sAddr, sAddr);
         }
@@ -412,15 +412,14 @@ const ServerAddrInfo *ServerAddrRegistry::get(const struct sockaddr *pAddr,
             if (pAddr->sa_family == AF_INET6)
             {
                 snprintf(sAddr, sizeof(sAddr), 
-                        "[::]:%d", GSockAddr::getPort(pAddr));
+                        "[::]:%u", (unsigned)GSockAddr::getPort(pAddr));
             }
             else
             {
-                snprintf(sAddr, sizeof(sAddr), "*:%d", 
-                        GSockAddr::getPort(pAddr));
+                snprintf(sAddr, sizeof(sAddr), "*:%u",
+                         (unsigned)GSockAddr::getPort(pAddr));
             }
             pListener = m_pListenerList->get(sAddr, sAddr);
-            assert(pListener != NULL);
         }
         
         if (pListener)
