@@ -210,10 +210,13 @@ private:
 
     void updateSSLEvent();
     void checkSSLReadRet(int ret);
-    void flushSslWpending();
+    void setAllowWrite();
+    int  flushSslWpending();
 
     int setupHandler(HiosProtocol verSpdy);
     int sslSetupHandler();
+    void enableTlsAccel();
+    void releaseIdleSslBuffer();
 
     void dumpState(const char *pFuncName, const char *action);
 
@@ -268,7 +271,7 @@ public:
     int writev_internal(const struct iovec *vector, int len, int flush_flag);
     int writev(const struct iovec *vector, int len);
 
-    int sendfile(int fdSrc, off_t off, off_t size);
+    int sendfile(int fdSrc, off_t off, size_t size, int flag);
 
     int addAioSFJob(Aiosfcb *cb);
     int aiosendfiledone(Aiosfcb *cb);
@@ -333,9 +336,9 @@ public:
     //void stopThrottleTimer();
     //void startThrottleTimer();
 
-    
+
     ThrottleControl *getThrottleCtrl() const;
-    
+
     static void enableThrottle(int enable);
     int isThrottle() const
     {   return m_pFpList->m_onTimer_fp != onTimer_; }
