@@ -23,10 +23,11 @@
 #include <modsecurity/modsecurity.h>
 #include <modsecurity/transaction.h>
 #include <modsecurity/rules.h>
+class session;
 
 #define MNAME                       mod_security
 #define ModuleNameStr               "Mod_Security"
-#define VERSIONNUMBER               "1.1"
+#define VERSIONNUMBER               "1.2"
 
 #define MODULE_VERSION_INFO         ModuleNameStr " " VERSIONNUMBER
 
@@ -49,7 +50,9 @@ typedef struct msc_conf_t_{
 
 typedef struct ModData_t
 {
-    Transaction             *modsec_transaction;
+    Transaction            *modsec_transaction;
+    int8_t                  chkReqBody;
+    int8_t                  chkRespBody;
 } ModData;
 
 
@@ -74,7 +77,7 @@ void ls_modSecLogCb(void *_session, const void *data)
 }
 
 
-int releaseMData(void *data)
+static int releaseMData(void *data)
 {
     ModData *myData = (ModData *)data;
     if (myData)
@@ -355,6 +358,16 @@ static int createModData(lsi_param_t *rec, msc_conf_t *conf)
     }
 }
 
+static int isBypassCheck(lsi_session_t *session)
+{
+    char var[4] = {0};
+    int len = g_api->get_req_env(session,
+                                 "modsecurity", 11, var, 3);
+    if (len == 3 && strncasecmp(var, "off", 3) == 0)
+        return 1;
+    else
+        return 0;
+}
 
 static int UriMapHook(lsi_param_t *rec)
 {
@@ -373,6 +386,13 @@ static int UriMapHook(lsi_param_t *rec)
                    ModuleNameStr);
         return LSI_OK;
     }
+    
+    if (isBypassCheck(session))
+    {
+        g_api->log(session, LSI_LOG_DEBUG,  "[Module:%s] bypassed for serving "
+                   "from cache.\n", ModuleNameStr);
+        return LSI_OK;
+    }
 
     ModData *myData = (ModData *) g_api->get_module_data(session, &MNAME,
                       LSI_DATA_HTTP);
@@ -468,6 +488,7 @@ static int UriMapHook(lsi_param_t *rec)
     struct iovec iov_val[MAX_REQ_HEADERS_NUMBER];
     count = g_api->get_req_headers(session, iov_key, iov_val,
                                     MAX_REQ_HEADERS_NUMBER);
+
     for (int i = 0; i < count; ++i)
     {
         msc_add_n_request_header(myData->modsec_transaction,
@@ -484,27 +505,45 @@ static int UriMapHook(lsi_param_t *rec)
                     "msc_process_request_headers failed.\n", ModuleNameStr);
         return LSI_ERROR;
     }
+
+#if 0
+    if (qs_len == 5 && strcmp(qs, "12345") == 0)
+    {
+        g_api->set_status_code(rec->session, 500);
+        return LSI_ERROR;
+    }
+#endif    
     
     Rules *rules = myData->modsec_transaction->m_rules;
-    bool chkReqBody = rules->m_secRequestBodyAccess == CHECKBODYTRUE;
-    bool chkRespBody = rules->m_secResponseBodyAccess == CHECKBODYTRUE;
+    myData->chkReqBody = rules->m_secRequestBodyAccess == CHECKBODYTRUE;
+    myData->chkRespBody = rules->m_secResponseBodyAccess == CHECKBODYTRUE;
     g_api->log(session, LSI_LOG_DEBUG, "[Module:%s] RequestBodyAccess: %s "
                "ResponseBodyAccess: %s\n", ModuleNameStr,
-               chkReqBody ? "YES" : "NO",
-               chkRespBody ? "YES" : "NO");
+               myData->chkReqBody ? "YES" : "NO",
+               myData->chkRespBody ? "YES" : "NO");
     
+    if(myData->chkReqBody && rules->m_requestBodyLimit.m_value > 3000) //at least set limit to 3000
+    {
+        long reqbodySize = g_api->get_req_content_length(session);
+        if (reqbodySize > rules->m_requestBodyLimit.m_value)
+        {
+            myData->chkReqBody = false;
+            g_api->log(session, LSI_LOG_DEBUG,
+                       "[Module:%s] RequestBodyAccess disabled due to size %ld > %ld.",
+                       ModuleNameStr, reqbodySize,
+                       (long)rules->m_requestBodyLimit.m_value);
+        }
+    }
+
     int aEnableHkpt[4] = {LSI_HKPT_RCVD_RESP_HEADER,
-                          LSI_HKPT_HANDLER_RESTART, };
-    int arrCount = 2;
-    if (chkReqBody)
+                          LSI_HKPT_HANDLER_RESTART,
+                          LSI_HKPT_RCVD_REQ_BODY,
+                          LSI_HKPT_RCVD_RESP_BODY};
+    if (myData->chkReqBody)
     {
         g_api->set_req_wait_full_body(session);
-        aEnableHkpt[arrCount ++] = LSI_HKPT_RCVD_REQ_BODY;
     }
-    if (chkRespBody)
-        aEnableHkpt[arrCount ++] = LSI_HKPT_RCVD_RESP_BODY;
-
-    g_api->enable_hook(session, &MNAME, 1, aEnableHkpt, arrCount);
+    g_api->enable_hook(session, &MNAME, 1, aEnableHkpt, 4);
     return LSI_OK;
 }
 
@@ -531,34 +570,42 @@ static int reqBodyHook(lsi_param_t *rec)
     g_api->log(session, LSI_LOG_DEBUG,
                "[Module:%s] reqBodyHook entry, len: %ld.\n", ModuleNameStr, len);
     
-    if (len == 0)
-        return LSI_OK;
-
-    do
+    if (len > 0 && myData->chkReqBody)
     {
-        len = 0;
-        if ((pTmpBuf = g_api->acquire_body_buf_block(pBuf, offset, (int *)&len))
-            == NULL)
-            break;
+        do
+        {
+            len = 0;
+            if ((pTmpBuf = g_api->acquire_body_buf_block(pBuf, offset, (int *)&len))
+                == NULL)
+                break;
 
-        //g_api->log(session, LSI_LOG_DEBUG,
-        //           "[Module:%s] reqBodyHook data: %ld bytes.\n", ModuleNameStr, len);
-        
-        msc_append_request_body(myData->modsec_transaction,
-                                (const unsigned char *)pTmpBuf, (size_t)len);
-        int ret = process_intervention(myData->modsec_transaction, rec);
-        if (ret != STATUS_OK) {
-            g_api->log(session, LSI_LOG_DEBUG,
-                    "[Module:%s] reqBodyHook failed.\n", ModuleNameStr);
-            return LSI_ERROR;
+            //g_api->log(session, LSI_LOG_DEBUG,
+            //           "[Module:%s] reqBodyHook data: %ld bytes.\n", ModuleNameStr, len);
+            
+            msc_append_request_body(myData->modsec_transaction,
+                                    (const unsigned char *)pTmpBuf, (size_t)len);
+
+            int ret = process_intervention(myData->modsec_transaction, rec);
+            if (ret != STATUS_OK) {
+                g_api->log(session, LSI_LOG_DEBUG,
+                        "[Module:%s] reqBodyHook failed.\n", ModuleNameStr);
+                return LSI_ERROR;
+            }
+            offset += len;
         }
-        offset += len;
-    }
-    while (!g_api->is_body_buf_eof(pBuf, offset)); 
+        while (!g_api->is_body_buf_eof(pBuf, offset));
     
-    g_api->log(session, LSI_LOG_DEBUG,
+        g_api->log(session, LSI_LOG_DEBUG,
                "[Module:%s] reqBodyHook used %ld bytes of %ld\n", 
                ModuleNameStr, offset, len);
+        
+    }
+    else
+    {
+        g_api->log(session, LSI_LOG_DEBUG,
+               "[Module:%s] reqBodyHook bypass reqBody len %d.\n",
+               ModuleNameStr, len);
+    }
     
     g_api->log(session, LSI_LOG_DEBUG,
                "[Module:%s] reqBodyHook final body check.\n", ModuleNameStr);
@@ -586,6 +633,14 @@ static int respHeaderHook(lsi_param_t *rec)
         return LSI_OK;
     }
 
+    if (isBypassCheck(session))
+    {
+        myData->chkRespBody = false;
+        g_api->log(session, LSI_LOG_DEBUG,  "[Module:%s] bypassed for serving "
+                   "from static file cache.\n", ModuleNameStr);
+        return LSI_OK;
+    }
+    
     int count = g_api->get_resp_headers_count(rec->session);
     if (count >= MAX_RESP_HEADERS_NUMBER)
         g_api->log(rec->session, LSI_LOG_WARN,
@@ -615,6 +670,28 @@ static int respHeaderHook(lsi_param_t *rec)
         return LSI_ERROR;
     }
 
+    Rules *rules = myData->modsec_transaction->m_rules;
+    bool chkRespBody = rules->m_secResponseBodyAccess == CHECKBODYTRUE;
+    if(chkRespBody && rules->m_responseBodyLimit.m_value > 3000) //at least set limit to 3000
+    {
+        struct iovec iov[1] = {{NULL, 0}};
+        int iovCount = g_api->get_resp_header(session,
+                                              LSI_RSPHDR_CONTENT_LENGTH,
+                                              NULL, 0, iov, 1);
+        if (iovCount == 1 && iov[0].iov_len > 0)
+        {
+            long len = atol((char *)iov[0].iov_base);
+            if (len > rules->m_responseBodyLimit.m_value)
+            {
+                int disableHkpt = LSI_HKPT_RCVD_RESP_BODY;
+                g_api->enable_hook(session, &MNAME, 0, &disableHkpt, 1);
+                g_api->log(session, LSI_LOG_DEBUG,
+                       "[Module:%s] ResponseBodyAccess disabled due to size %ld > %ld.",
+                       ModuleNameStr, len,
+                       (long)rules->m_responseBodyLimit.m_value);
+            }
+        }
+    }
     return LSI_OK;
 }
 
@@ -639,7 +716,7 @@ static int respHeaderHook(lsi_param_t *rec)
     int len = 0;
     void *pRespBodyBuf = g_api->get_resp_body_buf(rec->session);
     int ret;
-    while (!g_api->is_body_buf_eof(pRespBodyBuf, offset))
+    while (myData->chkRespBody && !g_api->is_body_buf_eof(pRespBodyBuf, offset))
     {
         len = 0;
         pBuf = g_api->acquire_body_buf_block(pRespBodyBuf, offset, &len);
