@@ -13,6 +13,7 @@
 #include <poll.h>
 #include <time.h>
 #include <fcntl.h>
+#include <stdlib.h>
 
 #include <lsdef.h>
 #include <lsr/ls_pool.h>
@@ -20,9 +21,9 @@
 #include <sslpp/ls_fdbuf_bio.h>
 #include <openssl/bio.h>
 
-#ifdef RUN_TEST
+//#ifdef RUN_TEST
 #define DEBUGGING
-#endif
+//#endif
 
 #ifdef DEBUGGING
 
@@ -30,9 +31,9 @@
 #define DEBUG_MESSAGE(...)     printf(__VA_ARGS__);
 #define INFO_MESSAGE(...)      printf(__VA_ARGS__);
 #else
-#include <ls.h>
-#define DEBUG_MESSAGE(...)     LSI_DBG(NULL, __VA_ARGS__);
-#define INFO_MESSAGE(... )     LSI_INF(NULL, __VA_ARGS__);
+#include <lsr/ls_log.h>
+#define DEBUG_MESSAGE(...)     LSR_DBG_H(__VA_ARGS__);
+#define INFO_MESSAGE(... )     LSR_INFO(__VA_ARGS__);
 #endif
 
 #else
@@ -77,29 +78,46 @@ void *BIO_get_data(BIO *a);
 void ls_fdbuf_bio_init(ls_fdbio_data *fdbio)
 {
     memset(fdbio, 0, sizeof(*fdbio));
+    fdbio->m_rbuf_size = RBIO_BUF_SIZE;
 }
 
 
-
-
-static int bio_fd_free(BIO *a)
+int ls_fdbio_alloc_rbuff(ls_fdbio_data *fdbio, int size)
 {
-    ls_fdbio_data *fdbio;
-    DEBUG_MESSAGE("[BIO] bio_fd_free: %p\n", a);
-    if (a == NULL)
-        return 0;
-    fdbio = LS_FDBUF_FROM_BIO(a);
-    if (!fdbio)
-        return 1;
-    if (fdbio->m_rbioBuf)
+    DEBUG_MESSAGE("[FDBIO] alloc read buf %d\n", size);
+    assert(fdbio->m_rbuf_used == fdbio->m_rbuf_read);
+    if (fdbio->m_rbuf && fdbio->m_rbuf_size >= size)
+        return LS_OK;
+    void *buf = ls_palloc(size);
+    if (buf)
     {
-        ls_pfree(fdbio->m_rbioBuf);
-        fdbio->m_rbioBuf = NULL;
+        if (fdbio->m_rbuf)
+            ls_pfree(fdbio->m_rbuf);
+        fdbio->m_rbuf = (uint8_t *)buf;
+        fdbio->m_rbuf_size = size;
+        fdbio->m_rbuf_used = fdbio->m_rbuf_read = 0;
+        return LS_OK;
+    }
+    else
+    {
+        DEBUG_MESSAGE("Insufficient memory allocating rbuff %d bytes\n", size);
+        errno = ENOMEM;
+        return LS_FAIL;
     }
-    ls_fdbuf_bio_init(fdbio);
-    return 1;
 }
 
+
+void ls_fdbio_release_rbuff(ls_fdbio_data *fdbio)
+{
+    if (fdbio->m_rbuf == NULL)
+        return;
+    assert(fdbio->m_rbuf_used == fdbio->m_rbuf_read);
+    DEBUG_MESSAGE("[FDBIO] free rbuf %d\n", (int)fdbio->m_rbuf_size);
+    ls_pfree(fdbio->m_rbuf);
+    fdbio->m_rbuf = NULL;
+}
+
+
 static int bio_fd_read(BIO *b, char *out, int outl)
 {
     int ret = 0;
@@ -112,10 +130,10 @@ static int bio_fd_read(BIO *b, char *out, int outl)
     if ((out == NULL) || (!outl))
     {
         INFO_MESSAGE("[BIO] bio_fd_read NO BUFFER!!\n");
-        err = ENOMEM;
+        err = EINVAL;
         return -1;
     }
-    if (fdbio->m_rbioClosed)
+    if (fdbio->m_is_closed)
     {
         DEBUG_MESSAGE("[BIO] bio_fd_read: CLOSED ON PREVIOUS READ\n");
         errno = 0;
@@ -123,7 +141,7 @@ static int bio_fd_read(BIO *b, char *out, int outl)
     }
     while (total < outl)
     {
-        int buffered = fdbio->m_rbioBuffered - fdbio->m_rbioIndex;
+        int buffered = fdbio->m_rbuf_used - fdbio->m_rbuf_read;
         int rd_remaining = outl - total;
         int copy = 0;
         if (buffered)
@@ -131,8 +149,8 @@ static int bio_fd_read(BIO *b, char *out, int outl)
             copy = (buffered > rd_remaining) ? rd_remaining : buffered;
             DEBUG_MESSAGE("[BIO] bio_fd_read: Use buffered %d of %d\n",
                           copy, rd_remaining);
-            memcpy(&out[total], &fdbio->m_rbioBuf[fdbio->m_rbioIndex], copy);
-            fdbio->m_rbioIndex += copy;
+            memcpy(&out[total], &fdbio->m_rbuf[fdbio->m_rbuf_read], copy);
+            fdbio->m_rbuf_read += copy;
             total += copy;
             if (total >= outl)
             {
@@ -140,16 +158,17 @@ static int bio_fd_read(BIO *b, char *out, int outl)
                               total);
                 return total;
             }
-            fdbio->m_rbioBuffered = 0;
-            fdbio->m_rbioIndex = 0;
+            fdbio->m_rbuf_used = 0;
+            fdbio->m_rbuf_read = 0;
             rd_remaining = outl - total;
         }
-        if ((rd_remaining < fdbio->m_rbioBufSz) && (fdbio->m_rbioBuf))
+        if ((rd_remaining < fdbio->m_rbuf_size)
+            && (fdbio->m_rbuf || ls_fdbio_alloc_rbuff(fdbio, fdbio->m_rbuf_size) == LS_OK))
         {
             DEBUG_MESSAGE("[BIO] bio_fd_read: Read into buffer\n");
-            ret = ls_read(fd, fdbio->m_rbioBuf, fdbio->m_rbioBufSz);
-            if (ret < fdbio->m_rbioBufSz)
-                fdbio->m_rbioWaitEvent = 1;
+            ret = ls_read(fd, fdbio->m_rbuf, fdbio->m_rbuf_size);
+            if (ret < fdbio->m_rbuf_size)
+                fdbio->m_need_read_event = 1;
             buffered = ret;
         }
         else
@@ -157,13 +176,13 @@ static int bio_fd_read(BIO *b, char *out, int outl)
             DEBUG_MESSAGE("[BIO] bio_fd_read: Read into data\n");
             ret = ls_read(fd, out + total, rd_remaining);
             if (ret < rd_remaining)
-                fdbio->m_rbioWaitEvent = 1;
+                fdbio->m_need_read_event = 1;
             buffered = 0;
         }
         if (ret == 0)
         {
             DEBUG_MESSAGE("[BIO] bio_fd_read: CLOSED\n");
-            fdbio->m_rbioClosed = 1;
+            fdbio->m_is_closed = 1;
             errno = 0;
             return total;
         }
@@ -186,8 +205,8 @@ static int bio_fd_read(BIO *b, char *out, int outl)
         }
         if (buffered)
         {
-            fdbio->m_rbioBuffered = buffered;
-            fdbio->m_rbioIndex = 0;
+            fdbio->m_rbuf_used = buffered;
+            fdbio->m_rbuf_read = 0;
             DEBUG_MESSAGE("[BIO] bio_fd_read: Preserve read: %d\n", ret);
         }
         else
@@ -200,19 +219,17 @@ static int bio_fd_read(BIO *b, char *out, int outl)
 }
 
 
-#define LS_FDBIO_FULL           1
-#define LS_FDBIO_BUFFERING      2
 int ls_fdbio_alloc_wbuff(ls_fdbio_data *fdbio, int size)
 {
     DEBUG_MESSAGE("[FDBIO] alloc write buf %d\n", size);
     assert(fdbio->m_wbuf_used == 0);
     if (fdbio->m_wbuf && fdbio->m_wbuf_size >= size)
         return LS_OK;
-    void *buf = ls_palloc_slab(size);
+    void *buf = ls_palloc(size);
     if (buf)
     {
         if (fdbio->m_wbuf)
-            ls_pfree_slab(fdbio->m_wbuf, fdbio->m_wbuf_size);
+            ls_pfree(fdbio->m_wbuf);
         fdbio->m_wbuf = (uint8_t *)buf;
         fdbio->m_wbuf_size = size;
         fdbio->m_wbuf_sent = 0;
@@ -222,19 +239,14 @@ int ls_fdbio_alloc_wbuff(ls_fdbio_data *fdbio, int size)
 }
 
 
-int ls_fdbio_is_idle(ls_fdbio_data *fdbio)
-{
-    return fdbio->m_wbuf && fdbio->m_wbuf_used == 0;
-}
-
-
-void ls_fdbio_relelase_wbuff(ls_fdbio_data *fdbio)
+void ls_fdbio_release_wbuff(ls_fdbio_data *fdbio)
 {
     assert(fdbio->m_wbuf_used == 0);
-    DEBUG_MESSAGE("[FDBIO] free buf %d\n", (int)fdbio->m_wbuf_size);
     if (fdbio->m_wbuf == NULL)
         return;
-    ls_pfree_slab(fdbio->m_wbuf, fdbio->m_wbuf_size);
+    DEBUG_MESSAGE("[FDBIO] free wbuf %d\n", (int)fdbio->m_wbuf_size);
+    ls_pfree(fdbio->m_wbuf);
+    fdbio->m_wbuf_size = 0;
     fdbio->m_wbuf = NULL;
 }
 
@@ -250,7 +262,7 @@ static int ls_fdbio_combine_write(ls_fdbio_data *fdbio, int fd, const void *buf,
     iov[1].iov_len = num;
     int ret = ls_writev(fd, iov, 2);
     DEBUG_MESSAGE("[FDBIO] ls_writev(%d + %d) ret: %d\n",
-                  iov[0].iov_len, iov[1].iov_len, ret);
+                  (int)iov[0].iov_len, (int)iov[1].iov_len, ret);
     if (ret > 0)
     {
         if (ret >= (int)iov[0].iov_len)
@@ -277,12 +289,11 @@ static int ls_fdbio_combine_write(ls_fdbio_data *fdbio, int fd, const void *buf,
 }
 
 
+#define TLS_RECORD_MAX_SIZE 16413
 static int ls_fdbio_buff_write(ls_fdbio_data *fdbio, int fd, const void *buf, int num)
 {
     DEBUG_MESSAGE("[FDBIO] ls_fdbio_buff_write, to write: %d, used: %d, sent: %d\n",
            num, (int)fdbio->m_wbuf_used, (int)fdbio->m_wbuf_sent);
-    if (fdbio->m_flag & LS_FDBIO_FULL)
-        return 0;
     if (!fdbio->m_wbuf)
     {
         if (num < 4096)
@@ -298,43 +309,26 @@ static int ls_fdbio_buff_write(ls_fdbio_data *fdbio, int fd, const void *buf, in
             ret = ls_fdbio_combine_write(fdbio, fd, buf, num);
         else
             ret = ls_write(fd, buf, num);
-        if (ret >= num && fdbio->m_wbuf_size < 16413 * 4)
+        if (ret >= num && fdbio->m_wbuf_size < TLS_RECORD_MAX_SIZE * 4)
         {
-            int new_size = fdbio->m_wbuf_size + 16413;
-            if (new_size < 16413 * 2)
-                new_size = 16413 * 2;
+            int new_size = fdbio->m_wbuf_size + TLS_RECORD_MAX_SIZE;
+            if (new_size < TLS_RECORD_MAX_SIZE * 2)
+                new_size = TLS_RECORD_MAX_SIZE * 2;
             ls_fdbio_alloc_wbuff(fdbio, new_size);
         }
         return ret;
     }
     memmove(fdbio->m_wbuf + fdbio->m_wbuf_used, buf, num);
     fdbio->m_wbuf_used += num;
-    DEBUG_MESSAGE("[FDBIO] lstls_buff_write, to write: %d, finished: %ld, used: %d, sent: %d\n",
+    DEBUG_MESSAGE("[FDBIO] lstls_buff_write, to write: %d, finished: %d, used: %d, sent: %d\n",
            num, num, (int)fdbio->m_wbuf_used, (int)fdbio->m_wbuf_sent);
     return num;
 }
 
 
-int ls_fdbio_is_full(ls_fdbio_data *fdbio)
-{   return fdbio->m_flag & LS_FDBIO_FULL;      }
-
-
-void ls_fdbio_clear_buff_full(ls_fdbio_data *fdbio)
-{    fdbio->m_flag &= ~LS_FDBIO_FULL;    }
-
-
-void ls_fdbio_set_wbuff(ls_fdbio_data *fdbio, int dobuff)
-{
-    if (dobuff)
-        fdbio->m_flag |= LS_FDBIO_BUFFERING;
-    else
-        fdbio->m_flag &= ~LS_FDBIO_BUFFERING;
-}
-
-
 int ls_fdbio_flush(ls_fdbio_data *fdbio, int fd)
 {
-    if (fdbio->m_flag & LS_FDBIO_FULL)
+    if (fdbio->m_flag & LS_FDBIO_WBLOCK)
         return 0;
     int pending = fdbio->m_wbuf_used - fdbio->m_wbuf_sent;
     if (pending <= 0)
@@ -351,8 +345,8 @@ int ls_fdbio_flush(ls_fdbio_data *fdbio, int fd)
     }
     else
     {
-        DEBUG_MESSAGE("[FDBIO] partial write, mark buffer FULL.\n");
-        fdbio->m_flag |= LS_FDBIO_FULL;
+        DEBUG_MESSAGE("[FDBIO] partial write, mark WBLOCK.\n");
+        fdbio->m_flag |= LS_FDBIO_WBLOCK;
         if (ret > 0)
         {
             fdbio->m_wbuf_sent += ret;
@@ -374,6 +368,14 @@ static int bio_fd_write(BIO *b, const char *in, int inl)
     ls_fdbio_data *fdbio = LS_FDBUF_FROM_BIO(b);
 
     DEBUG_MESSAGE("[FDBIO] bio_fd_write: %p, %d bytes on %d\n", b, inl, fd);
+    if (fdbio->m_flag & LS_FDBIO_WBLOCK)
+    {
+        DEBUG_MESSAGE("[FDBIO] bio_fd_write, FDBIO_WBLOCK flag is set, set errno to EAGAIN\n");
+        BIO_set_retry_write(b);
+        errno = EAGAIN;
+        return -1;
+    }
+
     if (fdbio->m_flag & LS_FDBIO_BUFFERING)
     {
         ret = ls_fdbio_buff_write(fdbio, fd, in, inl);
@@ -553,19 +555,8 @@ BIO *ls_fdbio_create(int fd, ls_fdbio_data *fdbio)
         s_biom->destroy = bio_fd_free;
 #endif        
     }
-    if (!fdbio->m_rbioBufSz)
-        fdbio->m_rbioBufSz = RBIO_BUF_SIZE;
-    
-    if (!fdbio->m_rbioBuf
-        && !(fdbio->m_rbioBuf = (char *)ls_palloc(fdbio->m_rbioBufSz)))
-    {
-        DEBUG_MESSAGE("Insufficient memory allocating %d bytes\n",
-                      fdbio->m_rbioBufSz);
-        errno = ENOMEM;
-        return NULL;
-    }
-    fdbio->m_rbioBuffered = 0;
-    fdbio->m_rbioIndex = 0;
+    fdbio->m_rbuf_used = 0;
+    fdbio->m_rbuf_read = 0;
 
     BIO *bio = BIO_new(s_biom);
     if (!bio)
@@ -589,4 +580,20 @@ BIO *ls_fdbio_create(int fd, ls_fdbio_data *fdbio)
 }
 
 
+static int bio_fd_free(BIO *a)
+{
+    ls_fdbio_data *fdbio;
+    DEBUG_MESSAGE("[BIO] bio_fd_free: %p\n", a);
+    if (a == NULL)
+        return 0;
+    fdbio = LS_FDBUF_FROM_BIO(a);
+    if (!fdbio)
+        return 1;
+    fdbio->m_rbuf_used = fdbio->m_rbuf_read = 0;
+    ls_fdbio_release_rbuff(fdbio);
+    fdbio->m_wbuf_used = 0;
+    ls_fdbio_release_wbuff(fdbio);
+    ls_fdbuf_bio_init(fdbio);
+    return 1;
+}
 
